@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module contains utilities to work with config files. It also supports looking up
 files according to the Freedesktop.org (XDG) Base Dir specification.
@@ -19,7 +19,7 @@ if sys.version_info >= (2, 6):
 else:
 	import simplejson as json # extra dependency
 
-from zim.fs import isfile, isdir, Dir, FileNotFoundError
+from zim.fs import isfile, isdir, File, Dir, FileNotFoundError
 from zim.errors import Error
 from zim.parsing import TextBuffer, split_quoted_strings
 
@@ -224,43 +224,25 @@ def user_dirs():
 	return dirs
 
 
-def value_check_class(value, default):
-	'''Check function for setdefault() which implements the default
-	behavior by checking classes of the value and the default.
-	No need to specify this function directly as it is the default.
+def check_class_allow_empty(value, default):
+	'''Check function for setdefault() which ensures the value is of
+	the same class as the default but allows it to be empty. THis is
+	the same as the default behavior when "allow_empty" is True.
+
+	Only reason to use this function is for places where setdefault()
+	is called indirectly, e.g. with arguments from plugin preferences.
 	'''
 	klass = default.__class__
-
-	if issubclass(klass, basestring):
-		klass = basestring
-		if not value: # Special case to avoid empty string
-			return default
-
-	if klass is tuple and isinstance(value, list):
-		return tuple(value)
-	elif not isinstance(value, klass):
-		return default
-	else:
-		return value
-
-
-def value_allow_empty(value, default):
-	'''Check function for setdefault() which follows the default
-	behavior checking classes, but allows the configured value to
-	be empty.
-	'''
-	# Same as value_check_class() but without special case for basestring
-	klass = default.__class__
-
 	if issubclass(klass, basestring):
 		klass = basestring
 
-	if klass is tuple and isinstance(value, list):
-		return tuple(value)
-	elif not isinstance(value, klass):
-		return default
-	else:
+	if value in ('', None) or isinstance(value, klass):
 		return value
+	elif klass is tuple and isinstance(value, list):
+		# Special case because json does not know difference list or tuple
+		return tuple(value)
+	else:
+		raise AssertionError, 'should be of type: %s' % klass
 
 
 def value_is_coord(value, default):
@@ -278,7 +260,7 @@ def value_is_coord(value, default):
 	):
 		return value
 	else:
-		return default
+		raise AssertionError, 'should be coordinate (tuple of int)'
 
 
 class ListDict(dict):
@@ -291,6 +273,12 @@ class ListDict(dict):
 		self.order = []
 		self._modified = False
 
+	def copy(self):
+		'''Shallow copy'''
+		new = self.__class__()
+		new.update(self)
+		return new
+
 	@property
 	def modified(self):
 		'''Recursive property'''
@@ -326,14 +314,18 @@ class ListDict(dict):
 		dict.__delitem__(self, k)
 		self.order.remove(k)
 
+	def __iter__(self):
+		return iter(self.order)
+
 	def pop(self, k):
 		v = dict.pop(self, k)
 		self.order.remove(k)
+		return v
 
 	# Would expect that setdefault() triggers __setitem__
 	# but this seems not to be the case in the standard implementation
 	# And we added some extra functionality here
-	def setdefault(self, key, default, check=None):
+	def setdefault(self, key, default, check=None, allow_empty=False):
 		'''Like dict.setdefault() but with some extra restriction
 		because we assume un-safe user input. If no extra arguments
 		are given it will compare the classes of the set value and the
@@ -351,12 +343,28 @@ class ListDict(dict):
 
 		If 'check' is given and it is a function it will be used to
 		check the value in the dictionary if it exists. The check
-		function should have two arguments, first is the value to be
-		tested, the second is the default value given to setdefault().
-		The return value will be used in the config dict.
+		function gets the current value and the default value as
+		arguments. The function should raise an AssertionError when the
+		value is not ok. The return value of the function is used to
+		replace the current value, so the check function can coerce
+		values into the proper form (but don't use this to return a
+		default!).
+		( Note that 'assert' statements in the code can be removed
+		by code optimization, so explicitly call 'raise' to raise the
+		AssertionError. )
 
 		If 'check' is given and is a set the value will be tested
-		against this set.
+		against this set. If 'check' is a list or a tuple and the
+		default is not an int it is also considered a set.
+
+		If the default is an integer and 'check' is a tuple of two
+		integers, the check will be that the value is in this range.
+		(For compatibility with InputForm.add_inputs arguments.)
+
+		If 'allow_empty' is True values are also allowed to be empty
+		string or None. This is used for optional parameters in the
+		config. By default 'allow_empty' is False but it is set to
+		True implicitely when the default value is None or ''.
 		'''
 		if not key in self:
 			self.__setitem__(key, default)
@@ -364,10 +372,22 @@ class ListDict(dict):
 
 		if check is None:
 			assert not default is None, 'Bad practise to set default to None without check'
-			check = value_check_class
+			klass = default.__class__
+			if issubclass(klass, basestring):
+				klass = basestring
+			check = klass
+
+		if default in ('', None):
+			allow_empty = True
+
+		if allow_empty and self[key] in ('', None):
+			return self[key]
 
 		if isinstance(check, (type, types.ClassType)): # is a class
 			klass = check
+			if not (allow_empty and default in ('', None)):
+				assert isinstance(default, klass), 'Default does not have correct class'
+
 			if not isinstance(self[key], klass):
 				if klass is tuple and isinstance(self[key], list):
 					# Special case because json does not know difference list or tuple
@@ -379,9 +399,19 @@ class ListDict(dict):
 						'Invalid config value for %s: "%s" - should be of type %s',
 						key, self[key], klass)
 					self.__setitem__(key, default)
+			elif self[key] == '':
+					# Special case for empty string
+					logger.warn(
+						'Invalid config value for %s: "%s" - not allowed to be empty',
+						key, self[key])
+					self.__setitem__(key, default)
 			else:
 				pass # value is OK
-		elif isinstance(check, set):
+		elif isinstance(check, (set, list)) \
+		or (isinstance(check, tuple) and not isinstance(default, int)):
+			if not (allow_empty and default in ('', None)):
+				assert default in check, 'Default is not within allows set'
+
 			if not self[key] in check:
 				logger.warn(
 						'Invalid config value for %s: "%s" - should be one of %s',
@@ -389,19 +419,33 @@ class ListDict(dict):
 				self.__setitem__(key, default)
 			else:
 				pass # value is OK
-		else: # assume callable
-			v = check(self[key], default)
-			if isinstance(v, tuple) and isinstance(self[key], list) \
-			and v == tuple(self[key]):
-				# Special case because json does not know difference list or tuple
-				modified = self.modified
-				self.__setitem__(key, v)
-				self.set_modified(modified) # don't change modified state
-			elif v != self[key]:
+		elif isinstance(check, tuple) and isinstance(default, int):
+			assert len(check) == 2 \
+				and isinstance(check[0], int) \
+				and isinstance(check[1], int)
+			if not isinstance(self[key], int):
 				logger.warn(
-					'Invalid config value for %s: "%s"',
+					'Invalid config value for %s: "%s" - should be integer',
 					key, self[key])
+				self.__setitem__(key, default)
+			elif not check[0] <= self[key] <= check[1]:
+				logger.warn(
+					'Invalid config value for %s: "%s" - should be between %i and %i',
+					key, self[key], check[0], check[1])
+				self.__setitem__(key, default)
+			else:
+				pass # value is OK
+		else: # assume callable
+			modified = self.modified
+			try:
+				v = check(self[key], default)
 				self.__setitem__(key, v)
+				self.set_modified(modified)
+			except AssertionError, error:
+				logger.warn(
+					'Invalid config value for %s: "%s" - %s',
+					key, self[key], error.args[0])
+				self.__setitem__(key, default)
 
 		return self[key]
 
@@ -409,14 +453,14 @@ class ListDict(dict):
 		return self.order[:]
 
 	def items(self):
-		return map(lambda k: (k, self[k]), self.order)
+		return tuple(map(lambda k: (k, self[k]), self.order))
 
 	def set_order(self, order):
 		'''Change the order in which items are listed by setting a list
 		of keys. Keys not in the list are moved to the end. Keys that are in
 		the list but not in the dict will be ignored.
 		'''
-		order = order[:] # copy
+		order = list(order[:]) # copy and convert
 		oldorder = set(self.order)
 		neworder = set(order)
 		for k in neworder - oldorder: # keys not in the dict
@@ -433,6 +477,14 @@ class ConfigDict(ListDict):
 	These are represented as INI files where each sub-dict is a section.
 	Sections are auto-vivicated when getting a non-existing key.
 	Each section is in turn a ListDict.
+
+	By default sections will be merged if they have the same name.
+	Values that appear under the same section name later in the file
+	will overwrite values that appeared earlier.
+
+	As a special case we can support sections that repeat under the
+	same section name. To do this assign the section name a list
+	before parsing.
 	'''
 
 	def __getitem__(self, k):
@@ -441,6 +493,12 @@ class ConfigDict(ListDict):
 		return dict.__getitem__(self, k)
 
 	def parse(self, text):
+		'''Parse 'text' and set values based on this input
+		@param text: a string or a list of lines
+		'''
+		# Note that we explicitly do _not_ support comments on the end
+		# of a line. This is because "#" could be a valid character in
+		# a config value.
 		if isinstance(text, basestring):
 			text = text.splitlines(True)
 		section = None
@@ -451,6 +509,9 @@ class ConfigDict(ListDict):
 			elif line.startswith('[') and line.endswith(']'):
 				name = line[1:-1].strip()
 				section = self[name]
+				if isinstance(section, list):
+					section.append(ListDict())
+					section = section[-1]
 			elif '=' in line:
 				parameter, value = line.split('=', 1)
 				parameter = str(parameter.rstrip()) # no unicode
@@ -485,13 +546,24 @@ class ConfigDict(ListDict):
 			return json.loads('"%s"' % value.replace('"', '\\"')) # force string
 
 	def dump(self):
+		'''Returns a list of lines with text representation of the
+		dict. Used to write as a config file.
+		'''
 		lines = []
+		def dump_section(name, parameters):
+			lines.append('[%s]\n' % section)
+			for param, value in parameters.items():
+				lines.append('%s=%s\n' % (param, self._encode_value(value)))
+			lines.append('\n')
+
 		for section, parameters in self.items():
 			if parameters:
-				lines.append('[%s]\n' % section)
-				for param, value in parameters.items():
-					lines.append('%s=%s\n' % (param, self._encode_value(value)))
-				lines.append('\n')
+				if isinstance(parameters, list):
+					for param in parameters:
+						dump_section(section, param)
+				else:
+					dump_section(section, parameters)
+
 		return lines
 
 	def _encode_value(self, value):
@@ -500,9 +572,8 @@ class ConfigDict(ListDict):
 		elif value is True: return 'True'
 		elif value is False: return 'False'
 		elif value is None: return 'None'
-		elif isinstance(value, object) and value.__class__.__name__.endswith('Path'):
-			# Hack to avoid importing Path here to test isinstance
-			return value.name
+		elif hasattr(value, 'serialize_zim_config'):
+			return value.serialize_zim_config()
 		else:
 			return json.dumps(value, separators=(',',':'))
 				# specify separators for compact encoding
@@ -676,8 +747,9 @@ class HierarchicDict(object):
 
 	__slots__ = ('dict')
 
-	def __init__(self):
+	def __init__(self, defaults=None):
 		self.dict = {}
+		self.dict['__defaults__'] = defaults or {}
 
 	def __getitem__(self, k):
 		if not isinstance(k, basestring):
@@ -715,7 +787,10 @@ class HierarchicDictFrame(object):
 			if key in self.dict and k in self.dict[key]:
 				return self.dict[key][k]
 		else:
-			raise KeyError
+			if k in self.dict['__defaults__']:
+				return self.dict['__defaults__'][k]
+			else:
+				raise KeyError
 
 	def __setitem__(self, k, v):
 		if not self.key in self.dict:
