@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Module with basic filesystem objects.
 
@@ -124,15 +124,23 @@ import codecs
 import logging
 from StringIO import StringIO
 
-from zim.errors import Error
+from zim.errors import Error, TrashNotSupportedError
 from zim.parsing import url_encode, url_decode
 from zim.async import AsyncOperation, AsyncLock
 
-
-__all__ = ['Dir', 'File']
-
 logger = logging.getLogger('zim.fs')
 
+gio = None
+try:
+	import gio
+	if not gio.File.trash:
+		gio = None
+except ImportError:
+	pass
+
+if not gio:
+	logger.warn("Trashing of files not supported, could not import 'gio'")
+
 
 xdgmime = None
 mimetypes = None
@@ -146,6 +154,9 @@ except ImportError:
 	import mimetypes
 
 
+__all__ = ['Dir', 'File']
+
+
 IMAGE_EXTENSIONS = (
 	# Gleaned from gtk.gdk.get_formats()
 	'bmp', # image/bmp
@@ -373,6 +384,9 @@ class UnixPath(object):
 
 		self._set_path(path) # overloaded in WindowsPath
 
+	def serialize_zim_config(self):
+		return self.path
+
 	@staticmethod
 	def _parse_uri(uri):
 		# Spec is file:/// or file://host/
@@ -498,18 +512,6 @@ class UnixPath(object):
 			# different drive ?
 			return None
 
-	def rename(self, newpath):
-		# Using shutil.move instead of os.rename because move can cross
-		# file system boundries, while rename can not
-		logger.info('Rename %s to %s', self, newpath)
-		with FS.get_async_lock(self):
-			# Do we also need a lock for newpath (could be the same as lock for self) ?
-			# TODO: check against newpath existing and being a directory
-			newpath.dir.touch()
-			shutil.move(self.encodedpath, newpath.encodedpath)
-		FS.emit('path-moved', self, newpath)
-		self.dir.cleanup()
-
 	def ischild(self, parent):
 		'''Returns True if this path is a child path of parent'''
 		return self.path.startswith(parent.path + os.path.sep)
@@ -548,6 +550,35 @@ class UnixPath(object):
 			else: return mimetype or 'application/octet-stream'
 
 
+	def rename(self, newpath):
+		# Using shutil.move instead of os.rename because move can cross
+		# file system boundries, while rename can not
+		logger.info('Rename %s to %s', self, newpath)
+		with FS.get_async_lock(self):
+			# Do we also need a lock for newpath (could be the same as lock for self) ?
+			# TODO: check against newpath existing and being a directory
+			newpath.dir.touch()
+			shutil.move(self.encodedpath, newpath.encodedpath)
+		FS.emit('path-moved', self, newpath)
+		self.dir.cleanup()
+
+	def trash(self):
+		'''Trash a file or folder, returns boolean for success.
+		Raises a TrashNotSupportedError if trashing is not possible.
+		'''
+		if not gio:
+			raise TrashNotSupportedError, 'gio not imported'
+
+		if self.exists():
+			f = gio.File(uri=self.uri)
+			if not f.trash():
+				# FIXME is this how to catch gio.ERROR_NOT_SUPPORTED ?
+				raise TrashNotSupportedError, 'Trashing failed'
+			return True
+		else:
+			return False
+
+
 class WindowsPath(UnixPath):
 
 	def _set_path(self, path):
@@ -598,11 +629,14 @@ class Dir(Path):
 		'''Returns True if the dir exists and is actually a dir'''
 		return os.path.isdir(self.encodedpath)
 
-	def list(self):
+	def list(self, raw=False):
 		'''Returns a list of names for files and subdirectories.
 		Will not return names that could not be decoded properly and
 		will throw warnings if those are encountered.
 		Hidden files are silently ignored.
+
+		The argument 'raw' doesn't doe anything here. It is there for
+		compatibility with the FilteredDir interface.
 		'''
 		files = []
 		if ENCODING == 'mbcs':
@@ -636,6 +670,32 @@ class Dir(Path):
 		else:
 			return []
 
+	def walk(self, raw=True):
+		'''Generator that yields all files and folders below this dir
+		as objects.
+		'''
+		for name in self.list(raw=raw):
+			path = self.path + os.path.sep + name
+			if os.path.isdir(path):
+				dir = self.subdir(name)
+				yield dir
+				for child in dir.walk(raw=raw):
+					yield child
+			else:
+				yield self.file(name)
+
+	def get_file_tree_as_text(self, raw=True):
+		'''Returns an overview of files and folders below this dir
+		as text.
+		'''
+		text = ''
+		for child in self.walk(raw=raw):
+			path = child.relpath(self)
+			if isinstance(child, Dir):
+				path += '/'
+			text += path + '\n'
+		return text
+
 	def touch(self):
 		'''Create this dir and any parent directories that do not yet exist'''
 		try:
@@ -683,18 +743,35 @@ class Dir(Path):
 				lrmdir(os.path.join(root, name))
 
 	def file(self, path):
-		'''Returns a File object for a path relative to this directory'''
-		assert isinstance(path, (Path, basestring, list, tuple))
-		if isinstance(path, File):
-			file = path
-		elif isinstance(path, basestring):
-			file = File((self.path, path))
-		else:
-			file = File((self.path,) + tuple(path))
+		'''Returns a File object for a path relative to this directory
+
+		When 'path' is in fact a L{File} object already this method
+		still enforces is to be below this directory. So this method can
+		be used as check as well.
+
+		@param path: a (relative) file path as string, tuple or L{Path} object
+
+		@returns: a L{File} object
+
+		@raises PathLookupError: if the file is not below this dir
+		'''
+		file = self.resolve_file(path)
 		if not file.path.startswith(self.path):
 			raise PathLookupError, '%s is not below %s' % (file, self)
 		return file
 
+	def resolve_file(self, path):
+		'''Like L{file()} but allows the path to start with "../"'''
+		assert isinstance(path, (Path, basestring, list, tuple))
+		if isinstance(path, basestring):
+			return File((self.path, path))
+		elif isinstance(path, (list, tuple)):
+			return File((self.path,) + tuple(path))
+		elif isinstance(path, File):
+			return path
+		elif isinstance(path, Path):
+			return File(path.path)
+
 	def new_file(self, path):
 		'''Like file() but guarantees the file does not yet exist by adding
 		sequentional numbers if needed.
@@ -717,18 +794,36 @@ class Dir(Path):
 		return file
 
 	def subdir(self, path):
-		'''Returns a Dir object for a path relative to this directory'''
-		assert isinstance(path, (Path, basestring, list, tuple))
-		if isinstance(path, Dir):
-			dir = path
-		elif isinstance(path, basestring):
-			dir = Dir((self.path, path))
-		else:
-			dir = Dir((self.path,) + tuple(path))
+		'''Returns a Dir object for a path relative to this directory
+
+		When 'path' is in fact a L{Dir} object already this method
+		still enforces is to be below this directory. So this method can
+		be used as check as well.
+
+		@param path: a (relative) file path as string, tuple or L{Path} object
+
+		@returns: a L{Dir} object
+
+		@raises PathLookupError: if the subdir is not below this dir
+		'''
+
+		dir = self.resolve_dir(path)
 		if not dir.path.startswith(self.path):
 			raise PathLookupError, '%s is not below %s' % (dir, self)
 		return dir
 
+	def resolve_dir(self, path):
+		'''Like L{subdir()} but allows the path to start with "../"'''
+		assert isinstance(path, (Path, basestring, list, tuple))
+		if isinstance(path, basestring):
+			return Dir((self.path, path))
+		elif isinstance(path, (list, tuple)):
+			return Dir((self.path,) + tuple(path))
+		elif isinstance(path, Dir):
+			return path
+		elif isinstance(path, Path):
+			return Dir(path.path)
+
 
 def _glob_to_regex(glob):
 	glob = glob.replace('.', '\\.')
@@ -754,9 +849,13 @@ class FilteredDir(Dir):
 		else:
 			return True
 
-	def list(self):
+	def list(self, raw=False):
+		'''As Dir.list() but filteres the results with the preset
+		filter. If 'raw' is True filtering is disabled.
+		'''
 		files = Dir.list(self)
-		files = filter(self.filter, files)
+		if not raw:
+			files = filter(self.filter, files)
 		return files
 
 
