@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module contains the Gtk user interface for zim.
 The main widgets and dialogs are seperated out in sub-modules.
@@ -33,21 +33,23 @@ import zim
 from zim import NotebookInterface, NotebookLookupError
 from zim.fs import *
 from zim.fs import normalize_win32_share
-from zim.errors import Error
-from zim.notebook import Path, Page, PageNameError, \
-	resolve_default_notebook, get_notebook, get_notebook_list
+from zim.errors import Error, TrashNotSupportedError
+from zim.notebook import Path, Page
 from zim.stores import encode_filename
 from zim.index import LINK_DIR_BACKWARD
 from zim.config import data_file, config_file, data_dirs, ListDict, value_is_coord
 from zim.parsing import url_encode, URL_ENCODE_DATA, is_win32_share_re
-from zim.history import History, HistoryRecord
+from zim.history import History, HistoryPath
+from zim.templates import list_templates, get_template
 from zim.gui.pathbar import NamespacePathBar, RecentPathBar, HistoryPathBar
 from zim.gui.pageindex import PageIndex
 from zim.gui.pageview import PageView
 from zim.gui.widgets import ui_environment, gtk_window_set_default_icon, \
 	Button, MenuButton, \
 	Window, Dialog, \
-	ErrorDialog, QuestionDialog, FileDialog, ProgressBarDialog, MessageDialog
+	ErrorDialog, QuestionDialog, FileDialog, ProgressBarDialog, MessageDialog, \
+	PromptExistingFileDialog, \
+	scrolled_text_view
 from zim.gui.clipboard import Clipboard
 from zim.gui.applications import ApplicationManager, CustomToolManager
 
@@ -92,6 +94,7 @@ ui_actions = (
 	('open_document_root', 'gtk-open', _('Open _Document Root'), '', '', True), # T: Menu item
 	('open_document_folder', 'gtk-open', _('Open _Document Folder'), '', '', True), # T: Menu item
 	('attach_file', 'zim-attachment', _('Attach _File'), '', _('Attach external file'), False), # T: Menu item
+	('show_clean_notebook', None, _('_Cleanup Attachments'), '', '', False), # T: Menu item
 	('edit_page_source', 'gtk-edit', _('Edit _Source'), '', '', False), # T: Menu item
 	('show_server_gui', None, _('Start _Web Server'), '', '', True), # T: Menu item
 	('reload_index', None, _('Re-build Index'), '', '', False), # T: Menu item
@@ -170,10 +173,12 @@ ui_preferences = (
 	# key, type, category, label, default
 	('tearoff_menus', 'bool', 'Interface', _('Add \'tearoff\' strips to the menus'), False),
 		# T: Option in the preferences dialog
-	('toggle_on_ctrlspace', 'bool', 'Interface', _('Use <Ctrl><Space> to switch to the side pane\n(If disabled you can still use <Alt><Space>)'), False),
+	('toggle_on_ctrlspace', 'bool', 'Interface', _('Use <Ctrl><Space> to switch to the side pane'), False),
 		# T: Option in the preferences dialog
 		# default value is False because this is mapped to switch between
 		# char sets in certain international key mappings
+	('remove_links_on_delete', 'bool', 'Interface', _('Remove links when deleting pages'), True),
+		# T: Option in the preferences dialog
 )
 
 if ui_environment['platform'] == 'maemo':
@@ -258,6 +263,8 @@ class GtkInterface(NotebookInterface):
 	* close-page (page)
 	  Called when closing a page, typically just before a new page is opened
 	  and before closing the application
+	* new-window (window)
+	  Called when a new window is created, can be used as a hook by plugins
 	* preferences-changed
 	  Emitted after the user changed the preferences
 	  (typically triggered by the preferences dialog)
@@ -265,6 +272,8 @@ class GtkInterface(NotebookInterface):
 	  Emitted when the ui changed from read-write to read-only or back
 	* quit
 	  Emitted when the application is about to quit
+	* start-index-update
+	* end-index-update
 
 	Also see signals in zim.NotebookInterface
 	'''
@@ -273,9 +282,12 @@ class GtkInterface(NotebookInterface):
 	__gsignals__ = {
 		'open-page': (gobject.SIGNAL_RUN_LAST, None, (object, object)),
 		'close-page': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+		'new-window': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 		'preferences-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
 		'readonly-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
 		'quit': (gobject.SIGNAL_RUN_LAST, None, ()),
+		'start-index-update': (gobject.SIGNAL_RUN_LAST, None, ()),
+		'end-index-update': (gobject.SIGNAL_RUN_LAST, None, ()),
 	}
 
 	ui_type = 'gtk'
@@ -333,8 +345,9 @@ class GtkInterface(NotebookInterface):
 			if not self.preferences['GtkInterface'].get(type):
 				default = manager.get_default_helper(type)
 				if default:
-					self.preferences['GtkInterface'].setdefault(type, default.key)
+					self.preferences['GtkInterface'][type] = default.key
 				else:
+					self.preferences['GtkInterface'][type] = None
 					logger.warn('No helper application defined for %s', type)
 
 		self.mainwindow = MainWindow(self, fullscreen, geometry)
@@ -472,7 +485,10 @@ class GtkInterface(NotebookInterface):
 			return True # keep ticking
 
 		# older gobject version doesn't know about seconds
-		self._autosave_timer = gobject.timeout_add(5000, schedule_autosave)
+		self.preferences['GtkInterface'].setdefault('autosave_timeout', 10)
+		timeout = self.preferences['GtkInterface']['autosave_timeout'] * 1000 # s -> ms
+		self._autosave_timer = gobject.timeout_add(timeout, schedule_autosave)
+			# FIXME make this more intelligent
 
 		self._finalize_ui = True
 		for plugin in self.plugins:
@@ -648,6 +664,72 @@ class GtkInterface(NotebookInterface):
 					self.uimanager.remove_ui(id)
 				handler._ui_merge_ids = None
 
+	def populate_popup(self, name, menu):
+		'''Populate a popup menu from a popup defined in the uimanager
+
+		This effectively duplicated the menu items from a given popup
+		as defined in the uimanager to a given menu. The reason to do
+		this is to include a menu that is extendable for plugins etc.
+		into an existing popup menu. (Note that changes to the menu
+		as returned by uimanager.get_widget() are global.)
+
+		@param name: the uimanager popup name, e.g. "toolbar_popup" or
+		"page_popup"
+		@param menu: a gtk.Menu to be populated with the menu items
+
+		@raises ValueError: when 'name' does not exist
+		'''
+		# ... so we have to do our own XML parsing here :(
+		# but take advantage of nicely formatted line-based output ...
+		xml = self.uimanager.get_ui()
+		xml = [l.strip() for l in xml.splitlines()]
+
+		# Get slice of XML
+		start, end = None, None
+		for i, line in enumerate(xml):
+			if start is None:
+				if line.startswith('<popup name="%s">' % name):
+					start = i
+			else:
+				if line.startswith('</popup>'):
+					end = i
+					break
+
+		if start is None or end is None:
+			raise ValueError, 'No such popup in uimanager: %s' % name
+
+		# Parse items and add to menu
+		seen_item = False # use to track empty parts
+		for line in xml[start+1:end]:
+			if line.startswith('<separator'):
+				if seen_item:
+					item = gtk.SeparatorMenuItem()
+					menu.append(item)
+				seen_item = False
+			elif line.startswith('<menuitem'):
+				pre, post = line.split('action="', 1)
+				actionname, post = post.split('"', 1)
+				for group in self.uimanager.get_action_groups():
+					action = group.get_action(actionname)
+					if action:
+						item = action.create_menu_item()
+
+						# don't show accels in popups (based on gtk/gtkuimanager.c)
+						child = item.get_child()
+						if isinstance(child, gtk.AccelLabel):
+							child.set_property('accel-closure', None)
+
+						break
+				else:
+					raise AssertionError, 'BUG: could not find action for "%s"' % actionname
+				menu.append(item)
+				seen_item = True
+			elif line.startswith('<placeholder') \
+			or line.startswith('</placeholder'):
+				pass
+			else:
+				raise AssertionError, 'BUG: Could not parse: ' + line
+
 	def set_readonly(self, readonly):
 		if not self.readonly:
 			# Save any modification now - will not be allowed after switch
@@ -671,19 +753,38 @@ class GtkInterface(NotebookInterface):
 		config file. Each preference is a tuple consisting of:
 
 		* the key in the config file
-		* an option type (see Dialog.add_fields() for more details)
+		* an option type (see InputForm() for more details)
 		* a category (the tab in which the option will be shown)
 		* a label to show in the dialog
 		* a default value
 		'''
 		register = self.preferences_register
 		for p in preferences:
-			key, type, category, label, default = p
-			self.preferences[section].setdefault(key, default)
+			if len(p) == 5:
+				key, type, category, label, default = p
+				self.preferences[section].setdefault(key, default)
+				r = (section, key, type, label)
+			else:
+				key, type, category, label, default, check = p
+				self.preferences[section].setdefault(key, default, check=check)
+				r = (section, key, type, label, check)
+
 			# Preferences with None category won't be shown in the preferences dialog
 			if category:
 				register.setdefault(category, [])
-				register[category].append((section, key, type, label))
+				register[category].append(r)
+
+
+	def register_new_window(self, window):
+		'''Called by windows and dialog to register themselves with
+		the application. Used e.g. by plugins that want to add some
+		widget to specific windows.
+		'''
+		#~ print 'WINDOW:', window
+		self.emit('new-window', window)
+
+	def do_new_window(self, window):
+		pass # TODO: keep register of pageviews
 
 	def get_path_context(self):
 		'''Returns the current 'context' for actions that want a path to start
@@ -715,8 +816,15 @@ class GtkInterface(NotebookInterface):
 			# We are already intialized, so let another process handle it
 			if self.usedaemon:
 				from zim.daemon import DaemonProxy
+				if isinstance(notebook, basestring) \
+				and notebook.startswith('zim+') \
+				and '?' in notebook:
+					# Interwiki link with page name attached
+					notebook, pagename = notebook.split('?', 1)
+				else:
+					pagename = None
 				notebook = DaemonProxy().get_notebook(notebook)
-				notebook.present()
+				notebook.present(page=pagename)
 			else:
 				self.spawn(notebook)
 
@@ -780,12 +888,12 @@ class GtkInterface(NotebookInterface):
 			# T: Title of progressbar dialog
 		dialog.show_all()
 		self.notebook.index.ensure_update(callback=lambda p: dialog.pulse(p.name))
-		dialog.set_total(self.notebook.index.n_all_pages())
+		dialog.set_total(self.notebook.index.n_list_all_pages())
 		self.notebook.upgrade_notebook(callback=lambda p: dialog.pulse(p.name))
 		dialog.destroy()
 
 	def on_notebook_properties_changed(self, notebook):
-		has_doc_root = not notebook.get_document_root() is None
+		has_doc_root = not notebook.document_root is None
 		for action in ('open_document_root', 'open_document_folder'):
 			action = self.actiongroup.get_action(action)
 			action.set_sensitive(has_doc_root)
@@ -793,7 +901,7 @@ class GtkInterface(NotebookInterface):
 	def open_page(self, path=None):
 		'''Emit the open-page signal. The argument 'path' can either be a Page
 		or a Path object. If 'page' is None a dialog is shown
-		to specify the page. If 'path' is a HistoryRecord we assume that this
+		to specify the page. If 'path' is a HistoryPath we assume that this
 		call is the result of a history action and the page is not added to
 		the history. The original path object is given as the second argument
 		in the signal, so handlers can inspect how this method was called.
@@ -831,7 +939,7 @@ class GtkInterface(NotebookInterface):
 		parent = self.actiongroup.get_action('open_page_parent')
 		child = self.actiongroup.get_action('open_page_child')
 
-		if isinstance(path, HistoryRecord):
+		if isinstance(path, HistoryPath):
 			historyrecord = path
 			self.history.set_current(path)
 			back.set_sensitive(not path.is_first)
@@ -949,11 +1057,15 @@ class GtkInterface(NotebookInterface):
 			if '\n' in name:
 				name, _ = name.split('\n', 1)
 			name = self.notebook.cleanup_pathname(name.replace(':', ''), purge=True)
+		elif isinstance(name, Path):
+			name = name.name
+			name = self.notebook.cleanup_pathname(name, purge=True)
 		else:
 			name = self.notebook.cleanup_pathname(name, purge=True)
+
 		path = self.notebook.resolve_path(name)
 		page = self.notebook.get_new_page(path)
-		page.parse('plain', text)
+		page.parse('wiki', text) # FIXME format hard coded
 		self.notebook.store_page(page)
 		if open_page:
 			self.open_page(page)
@@ -961,9 +1073,11 @@ class GtkInterface(NotebookInterface):
 
 	def append_text_to_page(self, name, text):
 		'''Append text to an (exising) page'''
+		if isinstance(name, Path):
+			name = name.name
 		path = self.notebook.resolve_path(name)
 		page = self.notebook.get_page(path)
-		page.parse('plain', text, append=True)
+		page.parse('wiki', text, append=True) # FIXME format hard coded
 		self.notebook.store_page(page)
 
 	def open_new_window(self, page=None):
@@ -1089,7 +1203,7 @@ class GtkInterface(NotebookInterface):
 			# Ask regardless of update_links because it might very
 			# well be that the dialog thinks there are no links
 			# but they are simply not indexed yet
-			cont = QuestionDialog(dialog or self,
+			cont = QuestionDialog(self,
 				_('The index is still busy updating. Until this '
 				  'is finished links can not be updated correctly. '
 				  'Performing this action now could break links, '
@@ -1103,7 +1217,6 @@ class GtkInterface(NotebookInterface):
 
 		dialog = ProgressBarDialog(self, _('Updating Links'))
 			# T: Title of progressbar dialog
-		dialog.show_all()
 		callback = lambda p, **kwarg: dialog.pulse(p.name, **kwarg)
 
 		try:
@@ -1117,7 +1230,23 @@ class GtkInterface(NotebookInterface):
 			return True
 
 	def delete_page(self, path=None):
-		DeletePageDialog(self, path).run()
+		'''Trash page or show DeletePageDialog'''
+		if path is None:
+			path = self.get_path_context()
+			if not path: return
+
+		update_links = self.preferences['remove_links_on_delete']
+		dialog = ProgressBarDialog(self, _('Removing Links'))
+			# T: Title of progressbar dialog
+		callback = lambda p, **kwarg: dialog.pulse(p.name, **kwarg)
+		try:
+			self.notebook.trash_page(path, update_links, callback)
+		except TrashNotSupportedError, error:
+			dialog.destroy()
+			logger.info('Trash not supported: %s', error.msg)
+			DeletePageDialog(self, path).run()
+		else:
+			dialog.destroy()
 
 	def show_properties(self):
 		from zim.gui.propertiesdialog import PropertiesDialog
@@ -1157,8 +1286,35 @@ class GtkInterface(NotebookInterface):
 		self.open_page(self.notebook.get_page(self.page))
 
 	def attach_file(self, path=None):
+		'''Show the AttachFileDialog'''
 		AttachFileDialog(self, path=path).run()
 
+	def do_attach_file(self, path, file, force_overwrite=False):
+		'''Callback for AttachFileDialog and InsertImageDialog
+		When 'force_overwrite' is False the user will be prompted in
+		case the new file has the same name as an existing attachment.
+		Returns the (new) filename or None when the action was canceled.
+		'''
+		namechanged = False
+		dir = self.notebook.get_attachments_dir(path)
+		if dir is None:
+			raise Error, '%s does not have an attachments dir' % path
+
+		dest = dir.file(file.basename)
+		if dest.exists() and not force_overwrite:
+			dialog = PromptExistingFileDialog(self, dest)
+			dest = dialog.run()
+			if dest is None:
+				return None	# dialog was cancelled
+
+		file.copyto(dest)
+		return dest
+
+	def show_clean_notebook(self):
+		'''Show the CleanNotebookDialog'''
+		from zim.gui.cleannotebookdialog import CleanNotebookDialog
+		CleanNotebookDialog(self).run()
+
 	def open_file(self, file):
 		'''Open either a File or a Dir in the file browser'''
 		assert isinstance(file, (File, Dir))
@@ -1235,12 +1391,12 @@ class GtkInterface(NotebookInterface):
 			assert False, 'BUG: notebook has neither dir or file'
 
 	def open_document_root(self):
-		dir = self.notebook.get_document_root()
+		dir = self.notebook.document_root
 		if dir and dir.exists():
 			self.open_file(dir)
 
 	def open_document_folder(self):
-		dir = self.notebook.get_document_root()
+		dir = self.notebook.document_root
 		if dir is None:
 			return
 
@@ -1338,22 +1494,18 @@ class GtkInterface(NotebookInterface):
 		'''Show a progress bar while updating the notebook index.
 		Returns True unless the user cancelled the action.
 		'''
-		# First make the index stop updating
-		self.mainwindow.pageindex.disconnect_model()
+		self.emit('start-index-update')
 
-		# Update the model
 		index = self.notebook.index
 		if flush:
 			index.flush()
 
 		dialog = ProgressBarDialog(self, _('Updating index'))
 			# T: Title of progressbar dialog
-		dialog.show_all()
 		index.update(callback=lambda p: dialog.pulse(p.name))
 		dialog.destroy()
 
-		# And reconnect the model - flushing out any sync error in treemodel
-		self.mainwindow.pageindex.reload_model()
+		self.emit('end-index-update')
 
 		return not dialog.cancelled
 
@@ -1524,48 +1676,47 @@ class MainWindow(Window):
 			return True # Do not destroy - let close() handle it
 		self.connect('delete-event', do_delete_event)
 
-		vbox = gtk.VBox()
-		self.add(vbox)
+		# setup the window layout
+		from zim.gui.widgets import TOP, BOTTOM, TOP_PANE, LEFT_PANE
 
 		# setup menubar and toolbar
 		self.add_accel_group(ui.uimanager.get_accel_group())
 		self.menubar = ui.uimanager.get_widget('/menubar')
 		self.toolbar = ui.uimanager.get_widget('/toolbar')
 		self.toolbar.connect('popup-context-menu', self.do_toolbar_popup)
-		vbox.pack_start(self.menubar, False)
-		vbox.pack_start(self.toolbar, False)
+		self.add_bar(self.menubar, TOP)
+		self.add_bar(self.toolbar, TOP)
 
-		# split window in side pane and editor
-		self.hpane = gtk.HPaned()
-		self.hpane.set_position(175)
-		vbox.add(self.hpane)
-		self.sidepane = gtk.VBox(spacing=5)
-		self.hpane.add1(self.sidepane)
+		self.sidepane = self._zim_window_left # FIXME - get rid of sidepane attribute
 
 		self.sidepane.connect('key-press-event',
 			lambda o, event: event.keyval == KEYVAL_ESC
 				and self.toggle_sidepane())
 
-
 		self.pageindex = PageIndex(ui)
-		self.sidepane.add(self.pageindex)
+		self.add_tab(_('Index'), self.pageindex, LEFT_PANE) # T: Label for pageindex tab
 
-		vbox2 = gtk.VBox()
-		self.hpane.add2(vbox2)
+		def check_focus_sidepane(window, widget):
+			focus = widget == self.pageindex
+				# FIXME - what if we have more widgets in side pane ?
+			if not focus:
+				self.on_sidepane_lost_focus()
+
+		self.connect('set-focus', check_focus_sidepane)
 
 		self.pathbar = None
 		self.pathbar_box = gtk.HBox() # FIXME other class for this ?
 		self.pathbar_box.set_border_width(3)
-		vbox2.pack_start(self.pathbar_box, False)
+		self.add_widget(self.pathbar_box, TOP_PANE, TOP)
 
 		self.pageview = PageView(ui)
 		self.pageview.view.connect_after(
 			'toggle-overwrite', self.do_textview_toggle_overwrite)
-		vbox2.add(self.pageview)
+		self.add(self.pageview)
 
 		# create statusbar
 		hbox = gtk.HBox(spacing=0)
-		vbox.pack_start(hbox, False, True, False)
+		self.add_bar(hbox, BOTTOM)
 
 		self.statusbar = gtk.Statusbar()
 		if ui_environment['platform'] == 'maemo':
@@ -1651,14 +1802,22 @@ class MainWindow(Window):
 
 		space = gtk.gdk.unicode_to_keyval(ord(' '))
 		group = gtk.AccelGroup()
-		group.connect_group( # <Alt><Space>
-			space, gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE,
-			self.do_switch_focus)
 
+		self.ui.preferences['GtkInterface'].setdefault('toggle_on_altspace', False)
+		if self.ui.preferences['GtkInterface']['toggle_on_altspace']:
+			# Hidden param, disabled because it causes problems with
+			# several international layouts (space mistaken for alt-space,
+			# see bug lp:620315)
+			group.connect_group( # <Alt><Space>
+				space, gtk.gdk.MOD1_MASK, gtk.ACCEL_VISIBLE,
+				self.toggle_focus_sidepane)
+
+		# Toggled by preference menu, also causes issues with international
+		# layouts - esp. when switching input method on Ctrl-Space
 		if self.ui.preferences['GtkInterface']['toggle_on_ctrlspace']:
 			group.connect_group( # <Ctrl><Space>
 				space, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE,
-				self.do_switch_focus)
+				self.toggle_focus_sidepane)
 
 		self.add_accel_group(group)
 		self._switch_focus_accelgroup = group
@@ -1784,10 +1943,10 @@ class MainWindow(Window):
 		if show:
 			self.sidepane.set_no_show_all(False)
 			self.sidepane.show_all()
-			self.hpane.set_position(self.uistate['sidepane_pos'])
+			self._zim_window_left_pane.set_position(self.uistate['sidepane_pos'])
 			self.pageindex.grab_focus()
 		else:
-			self.uistate['sidepane_pos'] = self.hpane.get_position()
+			self.uistate['sidepane_pos'] = self._zim_window_left_pane.get_position()
 			self.sidepane.hide_all()
 			self.sidepane.set_no_show_all(True)
 			self.pageview.grab_focus()
@@ -1795,7 +1954,11 @@ class MainWindow(Window):
 		self._sidepane_autoclose = False
 		self.uistate['show_sidepane'] = show
 
-	def do_switch_focus(self, *a):
+	def toggle_focus_sidepane(self, *a):
+		'''Switch focus between the textview and the sidepane.
+		Automatically opens the sidepane if it is closed
+		(but sets a property to automatically close it again).
+		'''
 		action = self.actiongroup.get_action('toggle_sidepane')
 		if action.get_active():
 			# side pane open
@@ -1814,6 +1977,12 @@ class MainWindow(Window):
 
 		return True # we are called from an event handler
 
+	def on_sidepane_lost_focus(self):
+		action = self.actiongroup.get_action('toggle_sidepane')
+		if self._sidepane_autoclose and action.get_active():
+			# Sidepane open and should close automatic
+			self.toggle_sidepane(show=False)
+
 	def set_pathbar(self, style):
 		'''Set the pathbar. Style can be either PATHBAR_NONE,
 		PATHBAR_RECENT, PATHBAR_HISTORY or PATHBAR_PATH.
@@ -2009,7 +2178,7 @@ class MainWindow(Window):
 		w, h = self.get_size()
 		if not self._fullscreen:
 			self.uistate['windowsize'] = (w, h)
-		self.uistate['sidepane_pos'] = self.hpane.get_position()
+		self.uistate['sidepane_pos'] = self._zim_window_left_pane.get_position()
 
 	def do_textview_toggle_overwrite(self, view):
 		state = view.get_overwrite()
@@ -2173,21 +2342,22 @@ discarded, but you can restore the copy later.''')
 		ErrorDialog.run(self)
 		gobject.source_remove(id)
 
-
 class OpenPageDialog(Dialog):
 	'''Dialog to go to a specific page. Also known as the "Jump to" dialog.
 	Prompts for a page name and navigate to that page on 'Ok'.
 	'''
 
-	def __init__(self, ui, namespace=None):
+	def __init__(self, ui):
 		Dialog.__init__(self, ui, _('Jump to'), # T: Dialog title
 			button=(None, gtk.STOCK_JUMP_TO),
-			path_context = ui.page,
-			fields=[('name', 'page', _('Jump to Page'), None)] # T: Label for page input
+		)
+
+		self.add_form(
+			[('page', 'page', _('Jump to Page'), ui.page)] # T: Label for page input
 		)
 
 	def do_response_ok(self):
-		path = self.get_field('name')
+		path = self.form['page']
 		if path:
 			self.ui.open_page(path)
 			return True
@@ -2210,44 +2380,48 @@ class NewPageDialog(Dialog):
 				'Please note that linking to a non-existing page\n'
 				'also creates a new page automatically.'),
 			# T: Dialog text in 'new page' dialog
-			path_context=path,
-			fields=[
-				#~ ('namespace', 'namespace', _('Namespace'), path.parent), # T: Input label
-				('name', 'page', _('Page Name'), None), # T: Input label
-			],
 			help=':Help:Pages'
 		)
 
+		self.path = path or ui.page
+
+		key = self.path or ''
+		default = ui.notebook.namespace_properties[key]['template']
+		templates = list_templates('wiki')
+		if not default in templates:
+			templates.insert(0, default)
+
+		self.add_form([
+			('page', 'page', _('Page Name'), (path or ui.page)), # T: Input label
+			('template', 'choice', _('Page Template'), templates) # T: Choice label
+		])
+
+		self.form['template'] = default
+		self.form.widgets['template'].set_no_show_all(True) # TEMP: hide feature
+		self.form.widgets['template'].set_property('visible', False) # TEMP: hide feature
+
 		if subpage:
-			self.inputs['name'].force_child = True
+			self.form.widgets['page'].subpaths_only = True
 
-		#~ if subpage:
-			#~ self.inputs['name'].force_child = True
-			#~ self.inputs['namespace'].set_path(path)
-			#~ self.inputs['namespace'].set_sensitive(False)
-
-		#~ self.inputs['name'].grab_focus()
-
-		# FIXME using namespace here need integration between pagename
-		# and namespace widget
+		# TODO: reset default when page input changed
 
 	def do_response_ok(self):
-		path = self.get_field('name')
-		if path:
-			page = self.ui.notebook.get_page(path)
-			if page.hascontent or page.haschildren:
-				ErrorDialog(self, _('Page exists')+': %s' % page.name).run() # T: error message
-				return False
-
-			template = self.ui.notebook.get_template(page)
-			tree = template.process_to_parsetree(self.ui.notebook, page)
-			page.set_parsetree(tree)
-			self.ui.open_page(page)
-			self.ui.save_page() # Save new page directly
-			return True
-		else:
+		path = self.form['page']
+		if not path:
 			return False
 
+		page = self.ui.notebook.get_page(path)
+		if page.hascontent or page.haschildren:
+			raise Error, _('Page exists')+': %s' % page.name
+				# T: Error when creating new page
+
+		template = get_template('wiki', self.form['template'])
+		tree = template.process_to_parsetree(self.ui.notebook, page)
+		page.set_parsetree(tree)
+		self.ui.open_page(page)
+		self.ui.save_page() # Save new page directly
+		return True
+
 
 class SaveCopyDialog(FileDialog):
 
@@ -2317,34 +2491,41 @@ class MovePageDialog(Dialog):
 			raise AssertionError, 'Could not save page'
 			# assert statement could be optimized away
 
-		i = self.ui.notebook.index.n_list_links_to_tree(
-					self.path, zim.index.LINK_DIR_BACKWARD )
-
 		self.vbox.add(gtk.Label(_('Move page "%s"') % self.path.name))
 			# T: Heading in 'move page' dialog - %s is the page name
-		linkslabel = ngettext(
+
+		indexpath = self.ui.notebook.index.lookup_path(self.path)
+		if indexpath:
+			i = self.ui.notebook.index.n_list_links_to_tree(
+					indexpath, zim.index.LINK_DIR_BACKWARD )
+		else:
+			i = 0
+
+		label = ngettext(
 			'Update %i page linking to this page',
 			'Update %i pages linking to this page', i) % i
 			# T: label in MovePage dialog - %i is number of backlinks
 			# TODO update lable to reflect that links can also be to child pages
 		self.context_page = self.path.parent
-		self.add_fields([
+		self.add_form([
 			('parent', 'namespace', _('Namespace'), self.context_page),
 				# T: Input label for namespace to move a file to
-			('links', 'bool', linkslabel, True),
+			('update', 'bool', label),
 				# T: option in 'move page' dialog
 		])
 
 		if i == 0:
-			self.inputs['links'].set_active(False)
-			self.inputs['links'].set_sensitive(False)
+			self.form['update'] = False
+			self.form.widgets['update'].set_sensitive(False)
+		else:
+			self.form['update'] = True
 
 	def do_response_ok(self):
-		parent = self.get_field('parent')
-		links = self.get_field('links')
+		parent = self.form['parent']
+		update = self.form['update']
 		newpath = parent + self.path.basename
 		self.hide() # hide this dialog before showing the progressbar
-		ok = self.ui.do_move_page(self.path, newpath, update_links=links)
+		ok = self.ui.do_move_page(self.path, newpath, update)
 		if ok:
 			return True
 		else:
@@ -2365,40 +2546,49 @@ class RenamePageDialog(Dialog):
 		page = self.ui.notebook.get_page(self.path)
 		existing = (page.hascontent or page.haschildren)
 
-		i = self.ui.notebook.index.n_list_links_to_tree(
-					self.path, zim.index.LINK_DIR_BACKWARD )
-
 		self.vbox.add(gtk.Label(_('Rename page "%s"') % self.path.name))
 			# T: label in 'rename page' dialog - %s is the page name
-		linkslabel = ngettext(
+
+		indexpath = self.ui.notebook.index.lookup_path(self.path)
+		if indexpath:
+			i = self.ui.notebook.index.n_list_links_to_tree(
+					indexpath, zim.index.LINK_DIR_BACKWARD )
+		else:
+			i = 0
+
+		label = ngettext(
 			'Update %i page linking to this page',
 			'Update %i pages linking to this page', i) % i
 			# T: label in MovePage dialog - %i is number of backlinks
-			# TODO update lable to reflect that links can also be to child pages
-		self.add_fields([
-			('name', 'string', _('Name'), self.path.basename),
+			# TODO update label to reflect that links can also be to child pages
+
+		self.add_form([
+			('name', 'string', _('Name')),
 				# T: Input label in the 'rename page' dialog for the new name
-			('head', 'bool', _('Update the heading of this page'), existing),
+			('head', 'bool', _('Update the heading of this page')),
 				# T: Option in the 'rename page' dialog
-			('links', 'bool', linkslabel, True),
+			('update', 'bool', label),
 				# T: Option in the 'rename page' dialog
-		])
+		], {
+			'name': self.path.basename,
+			'head': existing,
+			'update': True,
+		})
 
 		if not existing:
-			self.inputs['head'].set_active(False)
-			self.inputs['head'].set_sensitive(False)
+			self.form['head'] = False
+			self.form.widgets['head'].set_sensitive(False)
 
 		if i == 0:
-			self.inputs['links'].set_active(False)
-			self.inputs['links'].set_sensitive(False)
+			self.form['update'] = False
+			self.form.widgets['update'].set_sensitive(False)
 
 	def do_response_ok(self):
-		name = self.get_field('name')
-		head = self.get_field('head')
-		links = self.get_field('links')
+		name = self.form['name']
+		head = self.form['head']
+		update = self.form['update']
 		self.hide() # hide this dialog before showing the progressbar
-		ok = self.ui.do_rename_page(
-			self.path, newbasename=name, update_heading=head, update_links=links)
+		ok = self.ui.do_rename_page(self.path, name, head, update)
 		if ok:
 			return True
 		else:
@@ -2423,7 +2613,7 @@ class DeletePageDialog(Dialog):
 		hbox.pack_start(img, False)
 
 		vbox = gtk.VBox(spacing=5)
-		hbox.add(vbox)
+		hbox.pack_start(vbox, False)
 
 		label = gtk.Label()
 		short = _('Delete page "%s"?') % self.path.basename
@@ -2433,14 +2623,19 @@ class DeletePageDialog(Dialog):
 		label.set_markup('<b>'+short+'</b>\n\n'+long)
 		vbox.pack_start(label, False)
 
-		i = self.ui.notebook.index.n_list_links_to_tree(
-					self.path, zim.index.LINK_DIR_BACKWARD )
-		linkslabel = ngettext(
+		indexpath = self.ui.notebook.index.lookup_path(self.path)
+		if indexpath:
+			i = self.ui.notebook.index.n_list_links_to_tree(
+					indexpath, zim.index.LINK_DIR_BACKWARD )
+		else:
+			i = 0
+
+		label = ngettext(
 			'Remove links from %i page linking to this page',
 			'Remove links from %i pages linking to this page', i) % i
 			# T: label in DeletePage dialog - %i is number of backlinks
 			# TODO update lable to reflect that links can also be to child pages
-		self.links_checkbox = gtk.CheckButton(label=linkslabel)
+		self.links_checkbox = gtk.CheckButton(label=label)
 		vbox.pack_start(self.links_checkbox, False)
 
 		if i == 0:
@@ -2449,20 +2644,36 @@ class DeletePageDialog(Dialog):
 		else:
 			self.links_checkbox.set_active(True)
 
+
+		# TODO use expander here
+		dir = self.ui.notebook.get_attachments_dir(self.path)
+		text = dir.get_file_tree_as_text(raw=True)
+		n = len([l for l in text.splitlines() if not l.endswith('/')])
+
+		string = ngettext('%i file will be deleted', '%i files will be deleted', n) % n
+			# T: label in the DeletePage dialog to warn user of attachments being deleted
+		if n > 0:
+			string = '<b>'+string+'</b>'
+
+		label = gtk.Label()
+		label.set_markup('\n'+string+':')
+		self.vbox.add(label)
+		window, textview = scrolled_text_view(text, monospace=True)
+		window.set_size_request(250, 200)
+		self.vbox.add(window)
+
 	def do_response_ok(self):
 		update_links = self.links_checkbox.get_active()
 
 		dialog = ProgressBarDialog(self, _('Removing Links'))
 			# T: Title of progressbar dialog
-		dialog.show_all()
 		callback = lambda p, **kwarg: dialog.pulse(p.name, **kwarg)
 
 		try:
 			self.ui.notebook.delete_page(self.path, update_links, callback)
 		except Exception, error:
-			ErrorDialog(self, error).run()
 			dialog.destroy()
-			return False
+			raise
 		else:
 			dialog.destroy()
 			return True
@@ -2480,8 +2691,8 @@ class AttachFileDialog(FileDialog):
 			self.path = path
 		assert self.path, 'Need a page here'
 
-		self.dir = self.ui.notebook.get_attachments_dir(self.path)
-		if self.dir is None:
+		dir = self.ui.notebook.get_attachments_dir(self.path)
+		if dir is None:
 			ErrorDialog(_('Page "%s" does not have a folder for attachments') % self.path)
 				# T: Error dialog - %s is the full page name
 			raise Exception, 'Page "%s" does not have a folder for attachments' % self.path
@@ -2503,15 +2714,16 @@ class AttachFileDialog(FileDialog):
 			# Similar code in zim.gui.InsertImageDialog
 
 		for file in files:
-			file.copyto(self.dir)
-			file = self.dir.file(file.basename)
+			file = self.ui.do_attach_file(self.path, file)
+			if file is None:
+				return False # Cancelled overwrite dialog
+
 			pageview = self.ui.mainwindow.pageview
 			if self.uistate['insert_attached_images'] and file.isimage():
-				try:
-					pageview.insert_image(file, interactive=False)
-				except:
-					logger.exception('Could not insert image')
-					pageview.insert_links([file]) # image type not supported?
+				ok = pageview.insert_image(file, interactive=False)
+				if not ok: # image type not supported?
+					logger.info('Could not insert image: %s', file)
+					pageview.insert_links([file])
 			else:
 				pageview.insert_links([file])
 
