@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module contains utilities to work with external applications
 it is based on the Freedesktop.org (XDG) Desktop Entry specification
@@ -313,7 +313,11 @@ class DesktopEntryDict(ConfigDict, Application):
 		if not icon:
 			return None
 
+		if isinstance(icon, File):
+			icon = icon.path
+
 		w, h = gtk.icon_size_lookup(size)
+
 		if '/' in icon or '\\' in icon:
 			if zim.fs.isfile(icon):
 				return gtk.gdk.pixbuf_new_from_file_at_size(icon, w, h)
@@ -412,6 +416,8 @@ class DesktopEntryDict(ConfigDict, Application):
 		elif value is False: return 'false'
 		elif isinstance(value, int) or isinstance(value, float):
 			return value.__str__()
+		elif isinstance(value, File):
+			return value.path # Icon can be file
 		else:
 			assert isinstance(value, basestring), 'Desktop files can not store complex data'
 			return json.dumps(value)[1:-1].replace('\\"', '"') # get rid of quotes
@@ -474,23 +480,22 @@ class NewApplicationDialog(Dialog):
 		Dialog.__init__(self, ui, _('Custom Command')) # T: Dialog title
 		self.apptype = type
 		self.mimetype = mimetype
-		self.add_fields( (
-			('name', 'string', _('Name'), ''), # T: Field in 'custom command' dialog
-			('exec', 'string', _('Command'), ''), # T: Field in 'custom command' dialog
+		self.add_form( (
+			('name', 'string', _('Name')), # T: Field in 'custom command' dialog
+			('exec', 'string', _('Command')), # T: Field in 'custom command' dialog
 		) )
 
 	def do_response_ok(self):
-		fields = self.get_fields()
-		if not (fields['name'] and fields['exec']):
+		name = self.form['name']
+		cmd = self.form['exec']
+		if not (name and cmd):
 			return False
 
 		manager = ApplicationManager()
 		if self.mimetype:
-			application = manager.create(
-				self.mimetype, fields['name'], fields['exec'] )
+			application = manager.create(self.mimetype, name, cmd)
 		else:
-			application = manager.create_helper(
-				self.apptype, fields['name'], fields['exec'] )
+			application = manager.create_helper(self.apptype, name, cmd)
 		self.result = application
 		return True
 
@@ -596,6 +601,7 @@ class CustomToolDict(DesktopEntryDict):
 		%n for notebook location (file or directory)
 		%D for document root
 		%t for selected text or word under cursor
+		%T for the selected text including wiki formatting
 
 	Other additional keys are:
 		X-Zim-ReadOnly				boolean
@@ -689,7 +695,7 @@ class CustomToolDict(DesktopEntryDict):
 			cmd[cmd.index('%n')] = File(notebook.uri).path
 
 		if '%D' in cmd:
-			dir = notebook.get_document_root()
+			dir = notebook.document_root
 			if dir:
 				cmd[cmd.index('%D')] = dir.path
 			else:
@@ -700,6 +706,11 @@ class CustomToolDict(DesktopEntryDict):
 			cmd[cmd.index('%t')] = text or ''
 			# FIXME - need to substitute this in arguments + url encoding
 
+		if '%T' in cmd:
+			text = pageview.get_selection(format='wiki') or pageview.get_word(format='wiki')
+			cmd[cmd.index('%T')] = text or ''
+			# FIXME - need to substitute this in arguments + url encoding
+
 		return tuple(cmd)
 
 	_cmd = parse_exec # To hook into Application.spawn and Application.run
